#!/usr/bin/env ruby

require 'rubygems'
require 'happymapper'  

# Module that deals with XML and Shell script mappings
module Mapping    
    
    # -- ---------------------------------------------------------------
    # -- Instance-level methods mixed into including classes
    # -- ---------------------------------------------------------------
    
    # Output object data to a bash compatible string       
    def to_bash(var=self.class.name.downcase)
        Mapping.map_to_bash(self, var)
    end           
	   
    # -- ---------------------------------------------------------------
    # -- Class-level methods mixed into including classes
    # -- ---------------------------------------------------------------
	
	# Class-level methods mixed into including classes
	module ClassMethods        
	   # Create a new object from the given xml and mapping
       def from_xml( xml, mapping )
           Mapping.map_from_xml( self, xml, mapping )
       end      
	
       # Define some pubblic attribute
       def def_public_attrs(*attrs)
           @public_attrs = [] if @public_attrs.nil?			   
		   attrs.each { |attr| def_public_attr attr }
       end
	   
       # Define a public attribute with ot without default accessors
	   def def_public_attr(attr, default=true)	       
           @public_attrs = [] if @public_attrs.nil?			   
		   if !@public_attrs.include? attr
                 @public_attrs << attr 
                 attr_accessor attr if default
		   end
	   end
	   
       # List public attributes
       def public_attrs
           @public_attrs
       end       
    end    
    
    # -- ---------------------------------------------------------------
    # -- Module-level methods
    # -- ---------------------------------------------------------------
  
    # Create a new object from the given class, xml and mapping
    def Mapping.map_from_xml( clazz, xml, mapping )
        require mapping unless mapping.nil?
        clazz.parse( xml, :single => true )
    end
		
    # Map object to a bash compatible string (note that arrays within arrays not supported)
    def Mapping.map_to_bash(obj, var)
        bash_s = ""
		obj_type = bash_type(obj)
        bash_s = case obj_type
		   when :mapper then map_bean_to_bash(obj, var)
		   when :array  then map_array_to_bash(obj, var)
		   when :scalar then map_scalar_to_bash(obj, var)
		   when :nil    then map_nil_to_bash(obj, var)
		   else ""
        end        
        bash_s    
    end
    
    # Map bean (that is an object with public attributes) to a bash compatible string 
    def Mapping.map_bean_to_bash(bean, var)
        return "" if bean.nil? # bean cannot be null
        return "" if var.nil?  # var cannot be null
        bash_s = ""       
        bean.class.public_attrs.each do |bean_attr|   
            bean_attr_val  = bean.send bean_attr
            bean_attr_type = bash_type(bean_attr_val)
            bean_attr_var  = "#{var}_#{bean_attr}"
            
            bash_s += case bean_attr_type
               when :mapper then bean_attr_val.to_bash(bean_attr_var) 
               when :scalar then map_scalar_to_bash( bean_attr_val, bean_attr_var )
               when :array  then map_array_to_bash( bean_attr_val, bean_attr_var )
               when :nil    then map_nil_to_bash( bean_attr_val, bean_attr_var )
			   else ""
            end
        end                        
        bash_s    
    end
   
    # Map array to a bash compatible string (note that arrays within arrays not supported)
    def Mapping.map_array_to_bash(arr, var)
       return "" if arr.nil?     # arr cannot be null
       return "" if var.nil?     # bash varname cannot be null
       
       bash_s = %Q{#{var}_length=#{arr.length}\n}
       arr.each_index do |arr_idx| 
          arr_item = arr[arr_idx]
	      arr_var = "#{var}#{arr_idx}"
          arr_item_type = bash_type(arr_item)
          bash_s += case arr_item_type
              when :mapper then arr_item.to_bash( arr_var )
              when :scalar then map_scalar_to_bash( arr_item, arr_var )
              when :array  then "" # array within arrays not supported
			  when :nil    then map_nil_to_bash( arr_item, arr_var )
              else ""
          end
       end
	   bash_s
    end           
           
    # Map a scalar (that is opaque) object to a Bash compatible string 
    def Mapping.map_scalar_to_bash(sca, var)
       return "" if sca.nil? # scalar cannot be null
       return "" if var.nil? # var cannot be null
       to_bash = %Q{#{var}="#{sca.to_s.strip}"\n}
	   to_bash
    end
	
    # Map nil to a Bash compatible string 
    def Mapping.map_nil_to_bash(obj, var)
       return "" unless obj.nil? # obj must be null
       return "" if var.nil? # var cannot be null
       nil_to_bash = %Q{#{var}=\n}
	   nil_to_bash
    end
    
    def Mapping.bash_type(obj)
	   return :nil    if obj.nil?
       return :mapper if obj.respond_to? :to_bash
       return :array  if obj.is_a? Array
       return :scalar
    end	
    
    # Callback when module is included
    def Mapping.included( mod )
        # Mixin ClassMethods into the class Mapping is included in
        mod.extend ClassMethods
    end        
        
end